module Solicit
  class PullRequestsController < ActionController::Base
    before_action :ensure_can_claim,  only: :claim
    before_action :ensure_can_notify, only: :notify

    def claim
      render json: ClaimPullRequest.perform(
        url: payload[:callback_id],
        label: payload.dig(:actions, 0, :value),
        assignee: Solicit.contributors_map[payload.dig(:user, :name)],
        original: payload[:original_message]
      )
    end

    def notify
      PostToSlack.perform(
        url: Solicit.slack_webhook_url,
        channel: Solicit.labels_map.stringify_keys[payload.dig(:label, :name).to_s],
        username: Solicit.slack_username,
        payload: payload
      )
      head :ok
    end

    private

    def ensure_can_claim
      head :ok unless request_is_from_slack? &&
                      payload.dig(:user, :name) &&
                      Solicit.contributors_map.key?(payload.dig(:user, :name)) &&
                      payload[:callback_id] &&
                      payload[:original_message] &&
                      payload.dig(:actions, 0, :value)
    end

    def ensure_can_notify
      label = payload.dig(:label, :name).to_s
      head :ok unless request_is_from_github? &&
                      payload[:action] == 'labeled' &&
                      Solicit.labels_map.stringify_keys.key?(label) &&
                      Solicit.needs_more_assignees.call(payload[:pull_request], label) &&
                      payload.dig(:pull_request, :number) &&
                      payload.dig(:repository, :url)
    end

    def request_is_from_slack?
      return true unless Solicit.slack_secret

      signature = 'v0=' + OpenSSL::HMAC.hexdigest(
        OpenSSL::Digest::SHA256.new,
        Solicit.slack_secret,
        ['v0', request.headers['X-Slack-Request-Timestamp'], request.body.read].join(':')
      )
      Rack::Utils.secure_compare(signature, request.headers['X-Slack-Signature'])
    end

    def request_is_from_github?
      return true unless Solicit.github_secret

      signature = 'sha1=' + OpenSSL::HMAC.hexdigest(
        OpenSSL::Digest.new('sha1'),
        Solicit.github_secret,
        request.body.read
      )
      Rack::Utils.secure_compare(signature, request.headers['HTTP_X_HUB_SIGNATURE'])
    end

    def payload
      @payload ||= JSON.parse(params.require(:payload)).with_indifferent_access
    end
  end
end
